autowatch = 1;
outlets = 4;
setoutletassist(0,"to lcd");
setoutletassist(1,"to MIDI learn");
setoutletassist(2,"-");
setoutletassist(3,"-");
var UI = new Object();
var hl = new Object();
var ol = new Object();
var currentparameter;
var DEBUG = 0;
var pname = this.patcher.name;

function loadbang(){
	hl = this.patcher.getnamed("mlearn_hlite");
	ol = this.patcher.getnamed("mlearn_overlay");	    	
	this.patcher.sendtoback(ol);
	this.patcher.sendtoback(hl);
}

function clear(){
	UI = new Object();
    outlet(0,"clear");
}
//create a database of objects for all UI elements so we can represent them graphically.
//properties are rect, MIDI, and type

//for testing - shows all the current objects in Max window
function dump(){
	for(i in UI){
		for(j in UI[i]){
			post("\n",i,j,UI[i][j]);
		}
	}
}

//from MIDI brain, lets us know what was just learned
function learned(){
	//argument example: EffectsMix::Effect[1] 176 8 32
	var a = arrayfromargs(arguments);
	var msg = a[0];
	var midi = a.slice(1);
	if(DEBUG) post("\nLearned-in",msg,midi,UI[msg]);
	//make sure the message is one that is used in here:
	if(UI[msg]){
		currentparameter = null;
		hl.hidden = 1;
		if(DEBUG) post("\nLearned",pname,msg,"midi",midi);
		UI[msg]["MIDI"] = midi;
		//need to update drawing for new label...
		draw();
	}
}

//from MIDI brain, clears the previous assignment for a MIDI message
function erase(){
	//argument example: EffectsMix::Effect[1] 176 8 32
	var a = arrayfromargs(arguments);
	var msg = a[0];
	//var midi = a.slice(1);
	if(DEBUG) post("\nClear Pvs",pname,msg);
	//send a message over to MIDI brain that this key and parameter need to be deleted?
	outlet(1,"deleteit",msg,UI[msg]["MIDI"]);
	UI[msg]["MIDI"] = null;
	//drawing will be updated by learned(), since that message will come next from MIDI brain.
	//draw();
}

function over(){ //mouse x,y in
	if(enabled){
		var x = arguments[0];
		var y = arguments[1];
		var hlwidth = 2;
		for(param in UI){
			//we need to cycle through all properties for each parameter in UI to get at the rectangle property:
			for(prop in UI[param]){
				//post("\n",UI[param]["rect"]);
				//get the rectangle values to see if the mouse intersects
				if(prop=="rect"){
					if(UI[param]["rect"] != null){
						rxo=UI[param]["rect"][0];
						ryo=UI[param]["rect"][1];
						rxf=rxo+UI[param]["rect"][2];
						ryf=ryo+UI[param]["rect"][3];
						//post("\ntest",param,x>rxo , y>ryo , x<rxf, y<ryf);
						//do the mouse coords fall into one of the zones?
						if(x>rxo && y>ryo && x<rxf && y<ryf){
							var hlrect = [rxo-hlwidth,ryo-hlwidth,rxf+hlwidth,ryf+hlwidth];
							//move the highlight rectangle so it's over this UI element:
							//for patching (non-presention) mode:
							hl.rect = hlrect;
							//for presenation mode:							
							hl.message("sendbox","presentation_rect",UI[param]["rect"]);
							if(currentparameter == param){
							 //turn off learn for the parameter if we press the area again
								hl.hidden = 1;
								outlet(1,"learn");//turn off learning in MIDI Brain.
							}else{
								//Match the parameter to a MIDI device:
								var rangescale = null;
								//unlikely we will define a range. default is 0.-1. for cc's, 0/1 for btns and togs
								if(UI[param]["range"]) rangescale = UI[param]["range"];
								var ctltype = UI[param]["type"];
								hl.hidden = 0;
								if(ctltype=="ignore"){
									//used so we can turn the MIDI Learn button off, but could be used for any button that you don't want in the MIDI Learn.
									var passthru = this.patcher.getnamed(param);
									passthru.message(0);
								}else{
									if(rangescale){
										outlet(1,"learn",param,ctltype,rangescale);
									}else{
										outlet(1,"learn",param,ctltype);
									}
								}
							}
							currentparameter = param; //for change detection
						}
					}
				}
			}
		}
	}
}

function deleteit(){
	if(currentparameter){
		erase(currentparameter);
		draw();
	}
}

var enabled=0;
function msg_int(v){
    outlet(0,"clear");
    currentparameter = null;
	hl.hidden=1; //don't turn it on just yet - wait until something is clicked. 
	ol.hidden=1-v;
	enabled=v;
	if(enabled){
		//make sure things are in the right place:
		ol.message("sendbox","presentation_position",0,0);
    	this.patcher.bringtofront(ol);
    	this.patcher.bringtofront(hl);
        draw();
    }
}

function draw(){	
	
    outlet(0,"clear");
	for(param in UI){
		if(UI[param]["rect"]){
			draw_element(param);
        }
	}
}

function draw_element(p){
	var sb;
	var overcolor = [0,128,0,128];
	var shape = "paintrect";
	var mtype = "--"; //midi type, such as: note, cc, pgm ch, etc.
	var asgn = "--";
	var tcolor = [255,255,255,255];
	var font = "Arial Black";
	var fsize = 10;
	var linespace = fsize-2;
	var x = UI[p]["rect"][0];
	var y = UI[p]["rect"][1];
	var w = UI[p]["rect"][2];
	var h = UI[p]["rect"][3];
	//we set this up so you can still click on the MIDI Learn button! In general, the "ignore" type used if a button is not MIDI-Learnable.
	var dodraw = 1;
	if(UI[p]["type"]=="ignore"){
		dodraw = 0;
	}
	if(UI[p]["MIDI"]){
		sb = UI[p]["MIDI"][0];
		mtype = miditype(sb);
		asgn = UI[p]["MIDI"][1]
	}
	//drawing commands for LCD:
	if(dodraw){
		outlet(0,"frgb",overcolor);
		outlet(0,shape,x,y,x+w,y+h);
		outlet(0,"font",font,fsize);
		outlet(0,"frgb",tcolor);
		//text coordinates:
		var tx = x+3;
		var ty = y+(h-2);
		if(w>37){
			outlet(0,"moveto",tx,ty);
			outlet(0,"write",mtype,asgn);
		}else{
			//write it on 2 lines for the narrow ones:
			outlet(0,"moveto",tx,ty-linespace);
			outlet(0,"write",mtype);
			outlet(0,"moveto",tx,ty);
			outlet(0,"write",asgn);
		}
	}
	
}

function miditype(v){
	var type;
	if(v>=128 && v<=143){ 
		type = "no"; //notes off <noteID&ch,note#,vel>
	}
	if(v>=144 && v<=159){ 
		type = "nn"; //notes <noteID&ch,note#,vel>
	}
	if(v>=160 && v<=175){
		type = "py"; //after(poly)touch <polyID&ch,note#,val>
	}
	if(v>=176 && v<=191){
		type = "cc"; //ctlr <ctlID&ch, cc#, val>
	}
	if(v>=192 && v<=207){ 
		type = "pg"; //pgm ch <pgmID&ch,val>
	}
	if(v>=208 && v<=223){
		type = "pr"; //ch. pressure <chprID&ch, val>
	}
	if(v>=224 && v<=239){
		type = "b"; //pitch bend <pbID&ch, msb, lsb>
	}
	return type;
}

function anything(){
	var a = arrayfromargs(messagename,arguments);
	var id = a[0];
	var property = a[1];
	var data = a.slice(2);
	//post("\n",id,",",property,",",data);
	if(UI[id]==null){
		UI[id]=new Object();
		//post("\nnew UI");
	}
	UI[id][property] = data;	
}